#!/usr/bin/perl
use warnings;
use strict;

# Usage: ./parse_fault_file.pl (directory path)

####### Variables ###################
my $num_metrics = 10;

my $path = $ARGV[0];
my $filename = $path."/data";
my %data;
my $max_value = 1e+10;
#####################################

# Load data file into data structure
my $run_id = 0;
#my $num_points=0;
my @num_points;
open FILE, $filename or die $!;
while (my $line = <FILE>) {
	chomp($line);
	
	# Split values
	my @tmp = ();
	@tmp = split(/\,/, $line);
	
	# number of data points per metric
	$num_points[$run_id] = @tmp / $num_metrics;
	
	print "Num_metrics=$num_metrics\n";
	my $measures = scalar @tmp;
	print "# measurements = $measures, Num_points = $num_points[$run_id]\n";
	
	for (my $i=0; $i < $num_metrics; $i++) {
		for (my $j=0; $j < $num_points[$run_id]; $j++) {
			$data{$run_id}{$i}{$j} = $tmp[($i * $num_points[$run_id]) + $j];
		}
	}
	
	$run_id++;
}
close(FILE);

# Saves last value per metric
# This is to eliminate buggy "> e+010" values.
my %lastValue;

# Write data into different files representing
# different runs of the fault injection
for (my $id=0; $id < $run_id; $id++) {
	my $file_id = $id + 1;
	my $out_file = $path."/run_"."$file_id";
	open FILE, ">$out_file" or die $!;
	for (my $j=0; $j < $num_points[$id]; $j++) {
		for (my $i=0; $i < $num_metrics; $i++) {
			
			# Eliminate buggy values
			my $tmp = $data{$id}{$i}{$j};
			if ($tmp > $max_value) {
				if (!$lastValue{$i}) {
					$data{$id}{$i}{$j} = 0;
				} else {
					$data{$id}{$i}{$j} = $lastValue{$i};
				}
			}
			
			print FILE "$data{$id}{$i}{$j}";
			
			# Save last value
			$lastValue{$i} = $data{$id}{$i}{$j};
			
			if ($i != ($num_metrics - 1)) {
				print FILE ",";
			} else {
				print FILE "\n";
			}
		}
	}
	close(FILE);
}
