% This script calculate a set of correlation-ceoficients vectors (CCVs)
% using the normal-behavior data set. This is done by sliding the
% obervation window by k steps.

% Load normal-behavior data file
%DATA = load('itap/itap_normal_traces_after_a_day_of_failure.dat');
%DATA = load('android_v3/normal_diff_v2.dat');
DATA = load('android_v4/normal_diff.dat');
DATA = DATA';

% number of workloads in the data file
%numberWorkloads = 3;
numberWorkloads = 1;

% size of workloads
sizeOfData = size(DATA);

% number of metrics
%numMetrics = 151;
numMetrics = 10; % android

% number of pair-wise coefficients
numCoeff = numMetrics*(numMetrics-1)/2;

% Observation window size
winSize = 100;

% Window slide size
%slideSize = 25;
slideSize = 1;

% size of workload
sizeOfWorkload = sizeOfData(1);

% Number of windows per workload
numWindows = floor((sizeOfWorkload - winSize) / slideSize);

% CCVs data base
CCV = zeros(numWindows*numberWorkloads,numCoeff);

% Standard-deviation vector
STD = load('STD.dat');

ccv_index = 1;
for i=1:numberWorkloads,
   %workload = zeros(sizeOfWorkload, numMetrics);
   workload = DATA;
   
   % Iterate over a sliding-window loop
   for n=1:numWindows,
       x = (n-1)*slideSize + 1; % lower bound
       y = x + winSize - 1; % upper bound
       block = workload(x:y,:);
       
       % Build CCVs
       %RHO = corr(block);
       RHO = pearson_corr(block, STD);
       %RHO = pearson_corr_scaled(block, STD);
       index = 1;
       for j=1:numMetrics-1,
            for k=j+1:numMetrics,
                if (isnan(RHO(j,k))),
                    CCV(ccv_index,index) = 0;
                else
                    CCV(ccv_index,index) = RHO(j,k);
                end
                index = index + 1;
            end
       end
       
       ccv_index = ccv_index + 1;
       
       if (mod(ccv_index,100) == 0)
            ccv_index
       end
   end
end

 save('CCV.dat','CCV','-ascii', '-double', '-tabs');
