function [RHO] = pearson_corr_scaled(data, STD)
% data: columns=metrics, rows=observations
% std: standard deviation per metric

sizeOfData = size(data);

% number of metrics
numMetrics = sizeOfData(2);

% number of pair-wise coefficients
%numCoeff = numMetrics*(numMetrics-1)/2;

% RHO matrix
RHO = zeros(numMetrics,numMetrics) + 1;

for i=1:numMetrics-1,
   for j=i+1:numMetrics,
       x = data(:,i);
       y = data(:,j);
       
       mean_x = mean(x);
       mean_y = mean(y);
       
       x = x - mean_x;
       y = y - mean_y;
       
       num = dot(x,y);
       
       % Calculate correlation coefficient
       coeff = num./(STD(i)*STD(j));
       
       % Calculate scaling factor
       std_x = std(x);
       std_y = std(y);
       factor = max((STD(i)/std_x),(STD(j)/std_y));
       factor = factor^2;
       
       % Scale correlation coefficient
       %if (factor < 1)
           coeff = coeff * factor;
       %end
       
       % Save coefficient
       RHO(i,j) = coeff;
       RHO(j,i) = coeff;
   end
end

end

