/*
 * arima_tools.h
 *
 *  Created on: Jun 8, 2010
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#ifndef ARIMA_TOOLS_H_
#define ARIMA_TOOLS_H_

#include <vector>

using namespace std;

typedef struct ARIMAModel_ {
	vector<double> ar_params;
	vector<double> ma_params;
	unsigned int  d;
	double intercept;
	double sigma_2;
	double aic;
} ARIMAModel;

typedef struct Predictions_ {
	vector<double> values;
	vector<double> error;
} Predictions;

typedef struct DiffSeries_ {
	vector< vector<double> > win;
} DiffSeries;

/*
 * Difference and Undo-difference a series
 */
class Series {
	DiffSeries dseries;
	unsigned int diff_d;
public:
	void diff(const vector<double> &old_series, vector<double> &new_series, unsigned int d);
	void undo_diff(vector<double> trans_predict, vector<double> &real_predict);
	void print(void);
};

/*
 * Predicts 'num' values using an ARIMA model
 */
void ARIMA_predict(ARIMAModel *model, unsigned int num, const vector<double> &values, Predictions *pred);

/*
 * Returns best model that fits observations (values).
 */
//void ARIMA_best_model(vector<double> values, ARIMAModel *model);


#endif /* ARIMA_TOOLS_H_ */
