/*
 * corr_coeff_program.cpp
 *
 *  Created on: Aug 25, 2010
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#include <vector>
#include <iostream>
#include <stdio.h>
#include <math.h>

using namespace std;

/*
 * This function returns a correlation-coefficient vector.
 * The size of the returned vector is (n-choose-2), where n=#metrics.
 *
 * The vector predict[i][j] that is as follows:
 * i: metric
 * j: prediction of metric i
 */
vector<double> getCorrCoeffVector(const vector< vector<double> > &data)
{
	vector<double> ret;
	vector<double> mu((int)(data.size()), 0);
	vector<double> sigma((int)(data.size()), 0);
	unsigned int num_metrics = data.size();
	unsigned int i,j,k;

	/*
	 * Calculate mu and sigma for each series
	 */
//#pragma omp parallel for private(i,j) shared(data,num_metrics,mu,sigma)
	for (i=0; i < num_metrics; i++) {
		for (j=0; j < data[i].size(); j++)
			mu[i] = mu[i] + data[i][j];
		mu[i] = mu[i] / (double)(data[i].size());

		for (j=0; j < data[i].size(); j++)
			sigma[i] = sigma[i] + pow((double)(data[i][j] - mu[i]), 2);
		sigma[i] = sigma[i] / (double)(data[i].size() - 1); // unbiased
		sigma[i] = sqrt(sigma[i]);
	}

	/*
	 * Calculate standard score for each value
	 */
	vector< vector<double> > std_score(num_metrics, vector<double>(data[0].size()));
//#pragma omp parallel for private(i,j) shared(data,num_metrics,mu,sigma,std_score)
	for (i=0; i < num_metrics; i++) {
		for (j=0; j < data[i].size(); j++) {
			if (sigma[i] > 0) {
				std_score[i][j] = (double)(data[i][j] - mu[i]) / sigma[i];
			} else {
				std_score[i][j] = 0;
			}
		}
	}

	/*
	 * Calculate the correlation-coefficient vector
	 */
	for (i=0; i < (num_metrics-1); i++) {
		for (j=i+1; j < num_metrics; j++) {
			double tmp = 0;
			for (k=0; k < std_score[i].size(); k++) {
				tmp = tmp + std_score[i][k] * std_score[j][k];
			}
			ret.push_back(tmp/(double)(std_score[i].size() - 1));
		}
	}

	return ret;
}

int main(void) {

	vector< vector<double> > data(5, vector<double>(6,0));

	/*
	 * Metric 1
	 */
	data[0][0] = 0;
	data[0][1] = 1;
	data[0][2] = 2;
	data[0][3] = 3;
	data[0][4] = 4;
	data[0][5] = 5;

	/*
	 * Metric 2
	 */
	data[1][0] = 3;
	data[1][1] = 5;
	data[1][2] = 2;
	data[1][3] = 8;
	data[1][4] = 4;
	data[1][5] = 9;

	/*
	 * Metric 3
	 */
	data[2][0] = 3;
	data[2][1] = 3;
	data[2][2] = 5;
	data[2][3] = 5;
	data[2][4] = 6;
	data[2][5] = 6;

	/*
	 * Metric 4
	 */
	data[3][0] = data[2][0];
	data[3][1] = data[2][1];
	data[3][2] = data[2][2];
	data[3][3] = data[2][3];
	data[3][4] = data[2][4];
	data[3][5] = data[2][5];

	/*
	 * Metric 5
	 */
	data[4][0] = 1;
	data[4][1] = 1;
	data[4][2] = 1;
	data[4][3] = 1;
	data[4][4] = 1;
	data[4][5] = 1;

	vector<double> ccv;
	ccv = getCorrCoeffVector(data);

	for (unsigned int i=0; i < ccv.size(); i++) {
		printf("%f, ", ccv[i]);
	}
	printf("\n");

	return 0;
}
