/*
 * correlation_coeff.C
 *
 *  Created on: Feb 10, 2011
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#include "correlation_coeff.h"
#include "global.h"
#include "utilities.h"
#include <math.h>
#include <stdlib.h>

#include <vector>

using namespace std;

vector<double> pearson_coefficient_vector(const vector< vector<double> > &data)
{
	//ASSERT_TRUE(data.size() == 151, "Data size == 151");
	//ASSERT_TRUE(data[0].size() == 100, "Blocks size == 100");

	//unsigned int metrics = modelsBase.size();
	unsigned int metrics = window[0].size();
	vector<double> ret;
	vector<double> mu(metrics, 0);
	vector<double> sigma(metrics, 0);
	unsigned int i,j,k;

	// Calculate mu and sigma for each series
	for (i=0; i < data.size(); i++) {
		for (j=0; j < data[i].size(); j++)
			mu[i] = mu[i] + data[i][j];
		mu[i] = mu[i] / (double)(data[i].size());

		/*for (j=0; j < data[i].size(); j++)
			sigma[i] = sigma[i] + pow((double)(data[i][j] - mu[i]), 2.0);
		sigma[i] = sigma[i] / (double)(data[i].size() - 1);
		sigma[i] = sqrt(sigma[i]);*/
		sigma[i] = standardDeviation[static_cast<size_t>(i)];
	}

	// Calculate standard score for each value
	vector< vector<double> > std_score(data.size(),
			vector<double>(data[0].size()));
	for (i=0; i < data.size(); i++) {
		for (j=0; j < data[i].size(); j++) {
			if (sigma[i] > 0)
				std_score[i][j] = (data[i][j] - mu[i]) / sigma[i];
			else
				std_score[i][j] = 0;
		}
	}

	// Calculate the correlation-coefficient vector
	for (i=0; i < (metrics-1); i++) {
		for (j=i+1; j < metrics; j++) {
			double tmp = 0;
			for (k=0; k < std_score[i].size(); k++)
				tmp = tmp + std_score[i][k] * std_score[j][k];
			ret.push_back(tmp/(double)(std_score[i].size() - 1.0));
		}
	}

	return ret;
}

vector<double> pearson_coefficient_vector_scaled(
		const vector< vector<double> > &data)
{
	//unsigned int metrics = modelsBase.size();
	unsigned int metrics = window[0].size();
	vector<double> ret;
	vector<double> mu(metrics, 0);
	vector<double> sigma(metrics, 0);
	vector<double> sigma_current(metrics, 0);
	unsigned int i,j,k;

	// Calculate mu and sigma for each series
	for (i=0; i < data.size(); i++) {
		for (j=0; j < data[i].size(); j++)
			mu[i] = mu[i] + data[i][j];
		mu[i] = mu[i] / (double)(data[i].size());

		for (j=0; j < data[i].size(); j++)
			sigma_current[i] += pow((double)(data[i][j] - mu[i]), 2.0);
		sigma_current[i] = sigma_current[i] / (double)(data[i].size() - 1);
		sigma_current[i] = sqrt(sigma_current[i]);

		sigma[i] = standardDeviation[static_cast<size_t>(i)];
	}

	// Calculate standard score for each value
	vector< vector<double> > std_score(data.size(),
			vector<double>(data[0].size()));
	for (i=0; i < data.size(); i++) {
		for (j=0; j < data[i].size(); j++) {
			if (sigma[i] > 0)
				std_score[i][j] = (data[i][j] - mu[i]) / sigma[i];
			else
				std_score[i][j] = 0;
		}
	}

	// Calculate the correlation-coefficient vector
	for (i=0; i < (metrics-1); i++) {
		for (j=i+1; j < metrics; j++) {
			double tmp = 0;
			for (k=0; k < std_score[i].size(); k++)
				tmp = tmp + std_score[i][k] * std_score[j][k];

			// Scale correlation coefficient (cc)
			double cc = tmp/(double)(std_score[i].size() - 1.0);
			double factor = max((sigma[i]/sigma_current[i]),
					(sigma[j]/sigma_current[j]));
			factor = pow(factor, 2.0);

			//if (factor < 1.0)
			cc = cc * factor;

			if (cc != cc) // check for NaN
				cc = 0;

			if (isinf(cc))
				cc = 100000;

			// Save cc
			ret.push_back(cc);
		}
	}

	return ret;
}

vector<double> kl_coefficient_vector(const vector< vector<double> > &data) {

	//unsigned int metrics = modelsBase.size();
	unsigned int metrics = window[0].size();
	vector<double> ret;
	vector<double> mu(metrics, 0);
	vector<double> sigma(metrics, 0);
	unsigned int i,j;

	// Calculate mu and sigma for each series
	for (i=0; i < data.size(); i++) {
		for (j=0; j < data[i].size(); j++)
			mu[i] = mu[i] + data[i][j];
		mu[i] = mu[i] / (double)(data[i].size());

		for (j=0; j < data[i].size(); j++)
			sigma[i] = sigma[i] + pow((double)(data[i][j] - mu[i]), 2.0);
		sigma[i] = sigma[i] / (double)(data[i].size() - 1);
		sigma[i] = sqrt(sigma[i]);

		// normalize them
		sigma[i] = sigma[i] / fabs((double)(mu[i]));
		//sigma[i] = pow(sigma[i], 2.0);

		// make sure sigmas are not zero
		//if (sigma[i] == 0) {
			//sigma[i] = 1e-6; // assign a small value
		//}
	}

	// Calculate symmetric KL-divergence
	for (i=0; i < (metrics-1); i++) {
		for (j=i+1; j < metrics; j++) {
			double kl_div=0;
			/*if (sigma[i] != 0 && sigma[j] != 0) {
				double div_1 = 0.5 * (sigma[i]/sigma[j] - 1.0 - log(sigma[i]/sigma[j]));
				double div_2 = 0.5 * (sigma[j]/sigma[i] - 1.0 - log(sigma[j]/sigma[i]));
				kl_div = 0.5 * div_1 + 0.5 * div_2;

				kl_div = fabs((double)(log(kl_div)));
			}*/

			if (sigma[i] > sigma[j]) {
				kl_div = sigma[j] / sigma[i];
			} else {
				kl_div = sigma[i] / sigma[j];
			}
			kl_div = fabs(log(kl_div));

			if (kl_div != kl_div) // check for NaN
				kl_div = 0;

			if (isinf(kl_div))
				kl_div = 1000;

			//ASSERT_TRUE(kl_div >= 0.0, "Divergence is non-negative");
			ret.push_back(kl_div);
		}
	}

	return ret;
}
