/*
 * correlation_coeff.h
 *
 *  Created on: Feb 10, 2011
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#ifndef CORRELATION_COEFF_H_
#define CORRELATION_COEFF_H_

#include <vector>

using namespace std;

/*
 * Correlation-coefficient vector (CCV) is built
 * using the Pearson correlation coefficient.
 *
 * IMPORTANT: the vector data[i][j] that is given as input is as follows:
 * i: metric
 * j: observation of metric i
 */
vector<double> pearson_coefficient_vector(
		const vector< vector<double> > &data);
vector<double> pearson_coefficient_vector_scaled(
		const vector< vector<double> > &data);

/*
 * Correlation-coefficient vector (CCV) is built
 * using the Spearman's rank correlation coefficient.
 *
 * IMPORTANT: the vector data[i][j] that is given as input is as follows:
 * i: metric
 * j: observation of metric i
 */
vector<double> spearman_coefficient_vector(
		const vector< vector<double> > &data);

/*
 * Correlation-coefficient vector (CCV) is built
 * using the symmetric KL-divergence (Jensen–Shannon divergence).
 *
 * IMPORTANT: the vector data[i][j] that is given as input is as follows:
 * i: metric
 * j: observation of metric i
 */
vector<double> kl_coefficient_vector(
		const vector< vector<double> > &data);

#endif /* CORRELATION_COEFF_H_ */
