/*
 * data_loading.C
 *
 *  Created on: Feb 9, 2011
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#include "data_loading.h"
#include "global.h"
#include "detection_engine.h"
#include "arima_tools.h"
#include "utilities.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <vector>
#include <string>
#include <iostream>
#include <fstream>

using namespace std;

void load_metric_names() {
	FILE *file_ptr;

	// Open fd
	string file_name = database_path + string("/metric_name_143.csv");
	if (!(file_ptr = fopen(file_name.c_str(), "r"))) {
		string msg("Error opening file: " + file_name);
		handle_error(msg.c_str());
	}

	size_t n = 500;
	char *line = (char *)malloc(n);
	line[0] = '\0'; // clean buffer
	ssize_t l;

	// Iterate over each line of the file
	if ((l = getline(&line, &n, file_ptr)) == -1)
		handle_error("Error reading line from file.");
	while(l > 0) {
		line[strlen(line)-1] = '\0';

		vector<string> tokens;
		Tokenize(string(line),tokens,",");

		unsigned int metric =
				static_cast<unsigned int>(atoi(tokens[0].c_str())) - 1;
		string name(tokens[1]);

		//metricNames[metric] = name;
		metricNames.insert(pair<unsigned int,string>(metric,name));

		l = getline(&line, &n, file_ptr);
	}
	fclose(file_ptr);

	//ASSERT_TRUE(metricNames.size() == 143, "Metric names table = 143");
}

void load_const_dims_vector() {

	FILE *file_ptr;

	// Open fd
	string file_name = database_path + string("/constant_metrics.dat");
	if (!(file_ptr = fopen(file_name.c_str(), "r"))) {
		string msg("Error opening file: " + file_name);
		handle_error(msg.c_str());
	}

	size_t n = 500;
	char *line = (char *)malloc(n);
	line[0] = '\0'; // clean buffer
	ssize_t l;

	// Iterate over each line of the file
	if ((l = getline(&line, &n, file_ptr)) == -1)
		handle_error("Error reading line from file.");
	while(l > 0) {

		if (line[0] != '#') { // Filter comment lines that start with '#'
			vector<string> tokens;
			Tokenize(string(line),tokens,",");

			const_metrics.push_back(
					static_cast<unsigned int>(atoi(tokens[1].c_str())));
			const_values.push_back(atof(tokens[2].c_str()));
		}

		l = getline(&line, &n, file_ptr);
	}
	fclose(file_ptr);
}

void loadValidCCVDimensions() {

	//ASSERT_TRUE(const_metrics.size() == config.num_metrics,
			//"Size of const_metrics == num_metrics");

	unsigned int count = 0;
	for (size_t i=0; i < config.num_metrics-1; i++) {
		for (size_t j=i+1; j < config.num_metrics; j++) {
			if (const_metrics[i] == 0 && const_metrics[j] == 0) {
				MetricPair pair;
				pair.x = static_cast<unsigned int>(i);
				pair.y = static_cast<unsigned int>(j);
				validCCVDimensions[count] = pair;
				count++;
			}
		}
	}

	//ASSERT_TRUE(h_sphere[0].size() == validCCVDimensions.size(),
			//"Size of an element of hyper_sphere == "
			//"size of valid CCV-dimensions map");
}

void load_ccvs()
{
	FILE *file_ptr;
	unsigned int i,j;

	// Open fd
	string file_name = database_path + string("/ccv_db_matlab_fixed_std.dat");
	//string file_name = database_path + string("/ccv_db_matlab_fixed_std_scaled.dat");
	//string file_name = database_path + string("/ccv_db_matlab_kl_div.dat");
	//string file_name = database_path + string("/ccv_db_matlab.dat");
	if (!(file_ptr = fopen(file_name.c_str(), "r"))) {
		string msg("Error opening file: " + file_name);
		handle_error(msg.c_str());
	}

	size_t n = 100000;
	char *line = (char *)malloc(n);
	line[0] = '\0'; // clean buffer
	ssize_t l;

	// Iterate over each line of the file
	if ((l = getline(&line, &n, file_ptr)) == -1)
		handle_error("Error reading line from file.");
	while(l > 0) {
		if (line[0] != '#') { // Filter comment lines that start with '#'
			vector<double> values;
			vector<string> tokens;
			Tokenize(string(line),tokens,",");

			unsigned int k=0;
			for (i=0; i < (config.num_metrics-1); i++) {
				for (j=i+1; j < config.num_metrics; j++) {
					if (const_metrics[i] == 0 && const_metrics[j] == 0) {
						double coeff = atof(tokens[k].c_str());
						values.push_back(coeff);
					}
					k++;
				}
			}

			h_sphere.push_back(values);
		}
		l = getline(&line, &n, file_ptr);
	}
	fclose(file_ptr);
	free(line);
}

void load_models()
{
	char line[500];
	line[0] = '\0';
	string models_path = database_path + string("/models");
	int c = 0;

	for (unsigned int i=1; i <= config.num_metrics; i++) {

		/* ONLY load model if metric is non-constant*/
		if (const_metrics[i] == 1) continue;

		/* Open files */
		char num[5];
		num[0] = '\0';
		sprintf(num, "%d", i);
		string name = string("/metric_") + string(num);
		string file_name = models_path + name;
		ifstream file(file_name.c_str());
		if (!file) {
			string msg(string("in load_models(): "
					"Could not open model file ") + file_name);
			handle_error(msg.c_str());
		}

		Models vec_models;
		unsigned int win = 1;
		while(file) {
			file.getline(line, 500);
			if (strlen(line) > 0) {

				/* Get model parameters */
				ARIMAModel model_tmp;

				vector<string> tokens;
				Tokenize(string(line),tokens,"|");

				vector<string> orders;
				Tokenize(tokens[0],orders,"/");

				if (!orders[0].compare("0") &&
						!orders[1].compare("0") &&
						!orders[2].compare("0")) {
					// Model of no use.
					// This is normally a model from a constant metric
					model_tmp.d = 0;
					model_tmp.ar_params.push_back(0);
					model_tmp.ma_params.push_back(0);
					model_tmp.intercept = 0;
					model_tmp.aic = 0;
					model_tmp.sigma_2 = 0;
				} else {
					vector<string> params;
					Tokenize(tokens[1],params,"/");

					if (atoi(orders[1].c_str()) == 0) { // d = 0
						int p = atoi(orders[0].c_str());
						int q = atoi(orders[2].c_str());
						for (int j=0; j < p; j++)
							model_tmp.ar_params.push_back(
									atof(params[j].c_str()));
						for (int j=p+0; j < q+p; j++)
							model_tmp.ma_params.push_back(
									atof(params[j].c_str()));
						model_tmp.d = 0;
						model_tmp.intercept = atof(params[p+q].c_str());
						model_tmp.aic = 0;
						model_tmp.sigma_2 = 0;
					} else { // d != 0
						int p = atoi(orders[0].c_str());
						int q = atoi(orders[2].c_str());
						for (int j=0; j < p; j++)
							model_tmp.ar_params.push_back(
									atof(params[j].c_str()));
						for (int j=p+0; j < q+p; j++)
							model_tmp.ma_params.push_back(
									atof(params[j].c_str()));
						model_tmp.d = (unsigned int)(atoi(orders[1].c_str()));
						model_tmp.intercept = 0;
						model_tmp.aic = 0;
						model_tmp.sigma_2 = 0;
					}
				}

				vec_models.mod.push_back(model_tmp);
				vec_models.num = win;
				win++;
			}
		}
		modelsBase.push_back(vec_models);
		file.close();
		c++;
	}

	//cout << "Effective number of metrics = " << num_effec_metrics << endl;
	//cout << "ModelBase size: " << modelsBase.size() << endl;
	//cout << "Params: " << modelsBase[142].mod[3].ar_params[1] << " " <<
	//modelsBase[142].mod[3].ma_params[1] << " "
	//<< modelsBase[142].mod[3].intercept << endl;
}

void load_standard_deviations() {
	FILE *file_ptr;

	// Open fd
	string file_name = database_path + string("/std_db_matlab.dat");
	if (!(file_ptr = fopen(file_name.c_str(), "r"))) {
		string msg("Error opening file: " + file_name);
		handle_error(msg.c_str());
	}

	size_t n = 500;
	char *line = (char *)malloc(n);
	line[0] = '\0'; // clean buffer
	ssize_t l;

	// Iterate over each line of the file
	if ((l = getline(&line, &n, file_ptr)) == -1)
		handle_error("Error reading line from file.");
	while(l > 0) {
		line[strlen(line)-1] = '\0';

		vector<string> tokens;
		Tokenize(string(line),tokens,",");

		for (size_t i=0; i < tokens.size(); i++) {
			if (const_metrics[static_cast<unsigned int>(i)] == 0) {
				double std = atof(tokens[i].c_str());
				standardDeviation.push_back(std);
			}
		}

		l = getline(&line, &n, file_ptr);
	}
	fclose(file_ptr);

	//ASSERT_TRUE(standardDeviation.size() == modelsBase.size(),
			//"Number of STDs is correct");
}
