/*
 * detection_engine.h
 *
 *  Created on: Jan 3, 2011
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#ifndef DETECTION_ENGINE_H_
#define DETECTION_ENGINE_H_

#define QUEUE_ID	111
#define MSG_SIZE	200

#include "arima_tools.h"

#include <time.h>
#include <sys/time.h>

#include <vector>
#include <string>
#include <map>

using namespace std;

/*------------------------------------------------------------*/
/* Global structures                                          */
/*------------------------------------------------------------*/

typedef struct {
	long type;
	double metrics[MSG_SIZE];
} MsgType;

typedef struct {
	unsigned int win_size; // window's size of observations
	unsigned int predict_size;
	bool use_predictions; // whether or not to use predictions
	unsigned int num_metrics;
} Config;

typedef struct {
	unsigned int x;
	unsigned int y;
} MetricPair;

/* ARIMA models database */
typedef struct Models_ {
	vector<ARIMAModel> mod; // ARIMA models per window
	unsigned int num; // Number of ARIMA models (or windows)
} Models;

/*------------------------------------------------------------*/
/* Global variables                                           */
/*------------------------------------------------------------*/

/* Configuration parameters */
//Config config;

/* ARIMA models database */
//vector<Models> modelsBase;

//string arima_tools_path = string("./arima_c_tools");
//string database_path; // Database root directory

/* Window of observations */
//vector< vector<double> > window;

//struct timeval curr_time;

/* Filtered constant dimensions (or metrics) */
//vector<unsigned int> const_metrics;
//vector<double> const_values;

/*
 * Map of CCVs valid dimensions and their corresponding pair of metrics.
 * A CCV dimension is valid only is both of the two metrics are non-constant.
 */
//map<unsigned int, MetricPair> validCCVDimensions;

/*
 * Nearest-Neighbor hyper-sphere.
 * Dimensions of h_sphere[x][y]:
 * x: window number (or sample id)
 * y: correlation-coefficient vector
 */
//vector< vector<double> > h_sphere;

/* Time measurements */
//map<string, vector<double> > time_traces;

/* Metric names */
//map<unsigned int, string> metricNames;

#endif /* DETECTION_ENGINE_H_ */
