/*
 * timing.C
 *
 *  Created on: Feb 9, 2011
 *      Author: Ignacio Laguna
 *     Contact: ilaguna@purdue.edu
 */

#include "timing.h"
#include "global.h"
#include "detection_engine.h"
#include <time.h>
#include <sys/time.h>

#include <map>
#include <vector>
#include <string>
#include <iostream>

using namespace std;

int timeval_subtract (struct timeval *result,
		struct timeval *x,
		struct timeval *y)
{
        /* Perform the carry for the later subtraction by updating y. */
        if (x->tv_usec < y->tv_usec) {
                int nsec = (y->tv_usec - x->tv_usec) / 1000000 + 1;
            y->tv_usec -= 1000000 * nsec;
            y->tv_sec += nsec;
        }
        if (x->tv_usec - y->tv_usec > 1000000) {
                int nsec = (x->tv_usec - y->tv_usec) / 1000000;
            y->tv_usec += 1000000 * nsec;
            y->tv_sec -= nsec;
        }

        /* Compute the time remaining to wait.
        tv_usec is certainly positive. */
        result->tv_sec = x->tv_sec - y->tv_sec;
        result->tv_usec = x->tv_usec - y->tv_usec;

        /* Return 1 if result is negative. */
        return x->tv_sec < y->tv_sec;
}

void get_time_start()
{
        gettimeofday(&curr_time, NULL);
        return;
}

void get_time_stop(const char *variable)
{
        struct timeval result;
        struct timeval t;
        double interval = 0;

        gettimeofday(&t, NULL);
        timeval_subtract(&result, &t, &curr_time);
        interval = (double)(result.tv_sec) +
        		(double)(result.tv_usec) / 1000000.0;

        //printf("Time: %f\n", interval);

        map<string, vector<double> >::iterator it =
        		time_traces.find(string(variable));
        if (it == time_traces.end()) {
        	vector<double> tmp;
        	tmp.push_back(interval);
        	time_traces[string(variable)] = tmp;
        } else {
        	it->second.push_back(interval);
        }

        return;
}

void print_times()
{
	if (time_traces.size() < 1) {
		//cout << "No traces to print out" << endl;
		return;
	}

	map<string, vector<double> >::const_iterator it;
	for (it = time_traces.begin(); it != time_traces.end(); it++) {
		cout << it->first << " "; // Name
		double avg = 0;
		for (size_t i=0; i < it->second.size(); i++) {
			avg += it->second.at(i);
		}
		avg = avg / static_cast<double>(it->second.size());
		cout << scientific << avg << endl;
	}

	return;
}
